@@ -1,7 +1,7 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2020 David R. MacIver
+# Most of this work is copyright (C) 2013-2021 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -47,6 +47,7 @@ class StoringReporter:
 # https://github.com/HypothesisWorks/hypothesis/issues/2490
 if tuple(map(int, pytest.__version__.split(".")[:2])) < (4, 3):  # pragma: no cover
     import warnings
+
     from hypothesis.errors import HypothesisWarning
 
     PYTEST_TOO_OLD_MESSAGE = """
@@ -85,17 +86,15 @@ else:
         )
 
     def pytest_report_header(config):
+        if config.option.verbose < 1 and settings.default.verbosity < Verbosity.verbose:
+            return None
         profile = config.getoption(LOAD_PROFILE_OPTION)
         if not profile:
             profile = settings._current_profile
         settings_str = settings.get_profile(profile).show_changed()
         if settings_str != "":
             settings_str = " -> %s" % (settings_str)
-        if (
-            config.option.verbose >= 1
-            or settings.default.verbosity >= Verbosity.verbose
-        ):
-            return "hypothesis profile %r%s" % (profile, settings_str)
+        return "hypothesis profile %r%s" % (profile, settings_str)
 
     def pytest_configure(config):
         core.running_under_pytest = True
@@ -169,7 +168,9 @@ else:
                         active_fx = item._request._get_active_fixturedef(fx.argname)
                         if active_fx.scope == "function":
                             note_deprecation(
-                                msg % (item.nodeid, fx.argname), since="2020-02-29"
+                                msg % (item.nodeid, fx.argname),
+                                since="2020-02-29",
+                                has_codemod=False,
                             )
 
             if item.get_closest_marker("parametrize") is not None:
