@@ -1,7 +1,7 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2020 David R. MacIver
+# Most of this work is copyright (C) 2013-2021 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -17,6 +17,7 @@ import enum
 import math
 import operator
 import random
+import re
 import string
 import sys
 import threading
@@ -25,6 +26,7 @@ from decimal import Context, Decimal, localcontext
 from fractions import Fraction
 from functools import reduce
 from inspect import Parameter, getfullargspec, isabstract, isclass, signature
+from types import FunctionType
 from typing import (
     Any,
     AnyStr,
@@ -283,7 +285,7 @@ def just(value: T) -> SearchStrategy[T]:
 
     Examples from this strategy do not shrink (because there is only one).
     """
-    return JustStrategy(value)
+    return JustStrategy([value])
 
 
 @defines_strategy(force_reusable_values=True)
@@ -298,12 +300,12 @@ def none() -> SearchStrategy[None]:
 
 @overload
 def one_of(args: Sequence[SearchStrategy[Any]]) -> SearchStrategy[Any]:
-    pass  # pragma: no cover
+    raise NotImplementedError
 
 
 @overload  # noqa: F811
 def one_of(*args: SearchStrategy[Any]) -> SearchStrategy[Any]:
-    pass  # pragma: no cover
+    raise NotImplementedError
 
 
 def one_of(*args):  # noqa: F811
@@ -339,12 +341,23 @@ def one_of(*args):  # noqa: F811
         # without incurring any performance overhead when there is only one
         # strategy, and keeps our reprs simple.
         return args[0]
+    if args and not any(isinstance(a, SearchStrategy) for a in args):
+        # And this special case is to give a more-specific error message if it
+        # seems that the user has confused `one_of()` for  `sampled_from()`;
+        # the remaining validation is left to OneOfStrategy.  See PR #2627.
+        raise InvalidArgument(
+            f"Did you mean st.sampled_from({list(args)!r})?  st.one_of() is used "
+            "to combine strategies, but all of the arguments were of other types."
+        )
     return OneOfStrategy(args)
 
 
 @cacheable
 @defines_strategy(force_reusable_values=True)
-def integers(min_value: int = None, max_value: int = None) -> SearchStrategy[int]:
+def integers(
+    min_value: Optional[int] = None,
+    max_value: Optional[int] = None,
+) -> SearchStrategy[int]:
     """Returns a strategy which generates integers.
 
     If min_value is not None then all values will be >= min_value. If
@@ -414,14 +427,14 @@ def booleans() -> SearchStrategy[bool]:
 @defines_strategy(force_reusable_values=True)
 @deprecated_posargs
 def floats(
-    min_value: Real = None,
-    max_value: Real = None,
+    min_value: Optional[Real] = None,
+    max_value: Optional[Real] = None,
     *,
-    allow_nan: bool = None,
-    allow_infinity: bool = None,
+    allow_nan: Optional[bool] = None,
+    allow_infinity: Optional[bool] = None,
     width: int = 64,
     exclude_min: bool = False,
-    exclude_max: bool = False
+    exclude_max: bool = False,
 ) -> SearchStrategy[float]:
     """Returns a strategy which generates floats.
 
@@ -441,8 +454,6 @@ def floats(
     required to represent the generated float. Valid values are 16, 32, or 64.
     Passing ``width=32`` will still use the builtin 64-bit ``float`` class,
     but always for values which can be exactly represented as a 32-bit float.
-    Half-precision floats (``width=16``) are not supported on Python 3.5,
-    unless :pypi:`Numpy` is installed.
 
     The exclude_min and exclude_max argument can be used to generate numbers
     from open or half-open intervals, by excluding the respective endpoints.
@@ -462,21 +473,16 @@ def floats(
 
     if allow_nan is None:
         allow_nan = bool(min_value is None and max_value is None)
-    elif allow_nan:
-        if min_value is not None or max_value is not None:
-            raise InvalidArgument(
-                "Cannot have allow_nan=%r, with min_value or max_value" % (allow_nan)
-            )
+    elif allow_nan and (min_value is not None or max_value is not None):
+        raise InvalidArgument(
+            "Cannot have allow_nan=%r, with min_value or max_value" % (allow_nan)
+        )
 
     if width not in (16, 32, 64):
         raise InvalidArgument(
             "Got width=%r, but the only valid values are the integers 16, "
             "32, and 64." % (width,)
         )
-    if width == 16 and sys.version_info[:2] < (3, 6) and "numpy" not in sys.modules:
-        raise InvalidArgument(  # pragma: no cover
-            "width=16 requires either Numpy, or Python >= 3.6"
-        )
 
     check_valid_bound(min_value, "min_value")
     check_valid_bound(max_value, "max_value")
@@ -647,13 +653,13 @@ def tuples(*args: SearchStrategy) -> SearchStrategy[tuple]:
 
 @overload
 def sampled_from(elements: Sequence[T]) -> SearchStrategy[T]:
-    pass  # pragma: no cover
+    raise NotImplementedError
 
 
 @overload  # noqa: F811
 def sampled_from(elements: Type[enum.Enum]) -> SearchStrategy[Any]:
     # `SearchStrategy[Enum]` is unreliable due to metaclass issues.
-    pass  # pragma: no cover
+    raise NotImplementedError
 
 
 @defines_strategy(try_non_lazy=True)  # noqa: F811
@@ -681,7 +687,7 @@ def sampled_from(elements):
         repr_ = "sampled_from(%s.%s)" % (elements.__module__, elements.__name__)
     else:
         repr_ = "sampled_from(%r)" % (elements,)
-    if hasattr(enum, "Flag") and isclass(elements) and issubclass(elements, enum.Flag):
+    if isclass(elements) and issubclass(elements, enum.Flag):
         # Combinations of enum.Flag members are also members.  We generate
         # these dynamically, because static allocation takes O(2^n) memory.
         # LazyStrategy is used for the ease of force_repr.
@@ -692,6 +698,10 @@ def sampled_from(elements):
     return SampledFromStrategy(values, repr_)
 
 
+def identity(x):
+    return x
+
+
 @cacheable
 @defines_strategy()
 @deprecated_posargs
@@ -699,9 +709,9 @@ def lists(
     elements: SearchStrategy[Ex],
     *,
     min_size: int = 0,
-    max_size: int = None,
-    unique_by: UniqueBy = None,
-    unique: bool = False
+    max_size: Optional[int] = None,
+    unique_by: Optional[UniqueBy] = None,
+    unique: bool = False,
 ) -> SearchStrategy[List[Ex]]:
     """Returns a list containing values drawn from elements with length in the
     interval [min_size, max_size] (no bounds in that direction if these are
@@ -739,9 +749,7 @@ def lists(
                 "(you probably only want to set unique_by)"
             )
         else:
-
-            def unique_by(x):
-                return x
+            unique_by = identity
 
     if max_size == 0:
         return builds(list)
@@ -759,6 +767,29 @@ def lists(
                 raise InvalidArgument("unique_by[%i]=%r is not a callable" % (i, f))
         # Note that lazy strategies automatically unwrap when passed to a defines_strategy
         # function.
+        tuple_suffixes = None
+        if (
+            # We're generating a list of tuples unique by the first element, perhaps
+            # via st.dictionaries(), and this will be more efficient if we rearrange
+            # our strategy somewhat to draw the first element then draw add the rest.
+            isinstance(elements, TupleStrategy)
+            and len(elements.element_strategies) >= 1
+            and len(unique_by) == 1
+            and (
+                # Introspection for either `itemgetter(0)`, or `lambda x: x[0]`
+                isinstance(unique_by[0], operator.itemgetter)  # type: ignore
+                and repr(unique_by[0]) == "operator.itemgetter(0)"
+                or isinstance(unique_by[0], FunctionType)
+                and re.fullmatch(
+                    get_pretty_function_description(unique_by[0]),
+                    r"lambda ([a-z]+): \1\[0\]",
+                )
+            )
+        ):
+            unique_by = (identity,)
+            tuple_suffixes = TupleStrategy(elements.element_strategies[1:])
+            elements = elements.element_strategies[0]
+
         if isinstance(elements, SampledFromStrategy):
             element_count = len(elements.elements)
             if min_size > element_count:
@@ -774,11 +805,19 @@ def lists(
                 max_size = element_count
 
             return UniqueSampledListStrategy(
-                elements=elements, max_size=max_size, min_size=min_size, keys=unique_by
+                elements=elements,
+                max_size=max_size,
+                min_size=min_size,
+                keys=unique_by,
+                tuple_suffixes=tuple_suffixes,
             )
 
         return UniqueListStrategy(
-            elements=elements, max_size=max_size, min_size=min_size, keys=unique_by
+            elements=elements,
+            max_size=max_size,
+            min_size=min_size,
+            keys=unique_by,
+            tuple_suffixes=tuple_suffixes,
         )
     return ListStrategy(elements, min_size=min_size, max_size=max_size)
 
@@ -787,7 +826,10 @@ def lists(
 @defines_strategy()
 @deprecated_posargs
 def sets(
-    elements: SearchStrategy[Ex], *, min_size: int = 0, max_size: int = None
+    elements: SearchStrategy[Ex],
+    *,
+    min_size: int = 0,
+    max_size: Optional[int] = None,
 ) -> SearchStrategy[Set[Ex]]:
     """This has the same behaviour as lists, but returns sets instead.
 
@@ -807,7 +849,10 @@ def sets(
 @defines_strategy()
 @deprecated_posargs
 def frozensets(
-    elements: SearchStrategy[Ex], *, min_size: int = 0, max_size: int = None
+    elements: SearchStrategy[Ex],
+    *,
+    min_size: int = 0,
+    max_size: Optional[int] = None,
 ) -> SearchStrategy[FrozenSet[Ex]]:
     """This is identical to the sets function but instead returns
     frozensets."""
@@ -828,7 +873,7 @@ class PrettyIter:
         return next(self._iter)
 
     def __repr__(self):
-        return "iter({!r})".format(self._values)
+        return f"iter({self._values!r})"
 
 
 @defines_strategy()
@@ -837,9 +882,9 @@ def iterables(
     elements: SearchStrategy[Ex],
     *,
     min_size: int = 0,
-    max_size: int = None,
-    unique_by: UniqueBy = None,
-    unique: bool = False
+    max_size: Optional[int] = None,
+    unique_by: Optional[UniqueBy] = None,
+    unique: bool = False,
 ) -> SearchStrategy[Iterable[Ex]]:
     """This has the same behaviour as lists, but returns iterables instead.
 
@@ -861,7 +906,7 @@ def iterables(
 def fixed_dictionaries(
     mapping: Dict[T, SearchStrategy[Ex]],
     *,
-    optional: Dict[T, SearchStrategy[Ex]] = None
+    optional: Optional[Dict[T, SearchStrategy[Ex]]] = None,
 ) -> SearchStrategy[Dict[T, Ex]]:
     """Generates a dictionary of the same type as mapping with a fixed set of
     keys mapping to strategies. ``mapping`` must be a dict subclass.
@@ -907,7 +952,7 @@ def dictionaries(
     *,
     dict_class: type = dict,
     min_size: int = 0,
-    max_size: int = None
+    max_size: Optional[int] = None,
 ) -> SearchStrategy[Dict[Ex, T]]:
     # Describing the exact dict_class to Mypy drops the key and value types,
     # so we report Dict[K, V] instead of Mapping[Any, Any] for now.  Sorry!
@@ -930,7 +975,7 @@ def dictionaries(
         tuples(keys, values),
         min_size=min_size,
         max_size=max_size,
-        unique_by=lambda x: x[0],
+        unique_by=operator.itemgetter(0),
     ).map(dict_class)
 
 
@@ -939,12 +984,12 @@ def dictionaries(
 @deprecated_posargs
 def characters(
     *,
-    whitelist_categories: Sequence[str] = None,
-    blacklist_categories: Sequence[str] = None,
-    blacklist_characters: Sequence[str] = None,
-    min_codepoint: int = None,
-    max_codepoint: int = None,
-    whitelist_characters: Sequence[str] = None
+    whitelist_categories: Optional[Sequence[str]] = None,
+    blacklist_categories: Optional[Sequence[str]] = None,
+    blacklist_characters: Optional[Sequence[str]] = None,
+    min_codepoint: Optional[int] = None,
+    max_codepoint: Optional[int] = None,
+    whitelist_characters: Optional[Sequence[str]] = None,
 ) -> SearchStrategy[str]:
     r"""Generates characters, length-one :class:`python:str`\ ings,
     following specified filtering rules.
@@ -1050,7 +1095,7 @@ def text(
     ),
     *,
     min_size: int = 0,
-    max_size: int = None
+    max_size: Optional[int] = None,
 ) -> SearchStrategy[str]:
     """Generates strings with characters drawn from ``alphabet``, which should
     be a collection of length one strings or a strategy generating such strings.
@@ -1141,7 +1186,11 @@ def from_regex(
 @cacheable
 @defines_strategy(force_reusable_values=True)
 @deprecated_posargs
-def binary(*, min_size: int = 0, max_size: int = None) -> SearchStrategy[bytes]:
+def binary(
+    *,
+    min_size: int = 0,
+    max_size: Optional[int] = None,
+) -> SearchStrategy[bytes]:
     """Generates :class:`python:bytes`.
 
     The generated :class:`python:bytes` will have a length of at least ``min_size``
@@ -1161,7 +1210,9 @@ def binary(*, min_size: int = 0, max_size: int = None) -> SearchStrategy[bytes]:
 @cacheable
 @defines_strategy()
 def randoms(
-    *, note_method_calls: bool = False, use_true_random: bool = None
+    *,
+    note_method_calls: bool = False,
+    use_true_random: Optional[bool] = None,
 ) -> SearchStrategy[random.Random]:
     """Generates instances of ``random.Random``. The generated Random instances
     are of a special HypothesisRandom subclass.
@@ -1184,6 +1235,7 @@ def randoms(
             want the new behaviour (which will become the default in future),
             set use_true_random=False.""",
             since="2020-06-30",
+            has_codemod=False,
         )
         use_true_random = True
 
@@ -1230,6 +1282,36 @@ def random_module() -> SearchStrategy[RandomSeeder]:
     return shared(RandomModule(), key="hypothesis.strategies.random_module()")
 
 
+class BuildsStrategy(SearchStrategy):
+    def __init__(self, target, args, kwargs):
+        self.target = target
+        self.args = args
+        self.kwargs = kwargs
+
+    def do_draw(self, data):
+        try:
+            return self.target(
+                *[data.draw(a) for a in self.args],
+                **{k: data.draw(v) for k, v in self.kwargs.items()},
+            )
+        except TypeError as err:
+            if (
+                isinstance(self.target, type)
+                and issubclass(self.target, enum.Enum)
+                and not (self.args or self.kwargs)
+            ):
+                name = self.target.__module__ + "." + self.target.__qualname__
+                raise InvalidArgument(
+                    f"Calling {name} with no arguments raised an error - "
+                    f"try using sampled_from({name}) instead of builds({name})"
+                ) from err
+            raise
+
+    def validate(self):
+        tuples(*self.args).validate()
+        fixed_dictionaries(self.kwargs).validate()
+
+
 # The ideal signature builds(target, /, *args, **kwargs) is unfortunately a
 # SyntaxError before Python 3.8 so we emulate it with manual argument unpacking.
 # Note that for the benefit of documentation and introspection tools, we set the
@@ -1238,7 +1320,7 @@ def random_module() -> SearchStrategy[RandomSeeder]:
 @defines_strategy()
 def builds(
     *callable_and_args: Union[Callable[..., Ex], SearchStrategy[Any]],
-    **kwargs: Union[SearchStrategy[Any], InferType]
+    **kwargs: Union[SearchStrategy[Any], InferType],
 ) -> SearchStrategy[Ex]:
     """Generates values by drawing from ``args`` and ``kwargs`` and passing
     them to the callable (provided as the first positional argument) in the
@@ -1296,11 +1378,10 @@ def builds(
             )
         for kw in set(hints) & (required | to_infer):
             kwargs[kw] = from_type(hints[kw])
+
     # Mypy doesn't realise that `infer` is gone from kwargs now
     # and thinks that target and args have the same (union) type.
-    return tuples(tuples(*args), fixed_dictionaries(kwargs)).map(  # type: ignore
-        lambda value: target(*value[0], **value[1])  # type: ignore
-    )
+    return BuildsStrategy(target, args, kwargs)
 
 
 if sys.version_info[:2] >= (3, 8):  # pragma: no cover
@@ -1447,33 +1528,51 @@ def _from_type(thing: Type[Ex]) -> SearchStrategy[Ex]:
         if getattr(thing, "__origin__", None) is typing.Union:
             args = sorted(thing.__args__, key=types.type_sorting_key)
             return one_of([from_type(t) for t in args])
-    # We can't resolve forward references, and under Python 3.5 (only)
-    # a forward reference is an instance of type.  Hence, explicit check:
-    elif type(thing) == getattr(typing, "_ForwardRef", None):  # pragma: no cover
-        raise ResolutionFailed(
-            "thing=%s cannot be resolved.  Upgrading to python>=3.6 may "
-            "fix this problem via improvements to the typing module." % (thing,)
-        )
     if not types.is_a_type(thing):
+        # The implementation of typing_extensions.Literal under Python 3.6 is
+        # *very strange*.  Notably, `type(Literal[x]) != Literal` so we have to
+        # use the first form directly, and because it uses __values__ instead of
+        # __args__ we inline the relevant logic here until the end of life date.
+        if types.is_typing_literal(thing):  # pragma: no cover
+            assert sys.version_info[:2] == (3, 6)
+            args_dfs_stack = list(thing.__values__)  # type: ignore
+            literals = []
+            while args_dfs_stack:
+                arg = args_dfs_stack.pop()
+                if types.is_typing_literal(arg):
+                    args_dfs_stack.extend(reversed(arg.__values__))
+                else:
+                    literals.append(arg)
+            return sampled_from(literals)
         raise InvalidArgument("thing=%s must be a type" % (thing,))
     # Now that we know `thing` is a type, the first step is to check for an
     # explicitly registered strategy.  This is the best (and hopefully most
     # common) way to resolve a type to a strategy.  Note that the value in the
     # lookup may be a strategy or a function from type -> strategy; and we
     # convert empty results into an explicit error.
-    if thing in types._global_type_lookup:
-        return as_strategy(types._global_type_lookup[thing], thing)
+    try:
+        if thing in types._global_type_lookup:
+            return as_strategy(types._global_type_lookup[thing], thing)
+    except TypeError:  # pragma: no cover
+        # This is due to a bizarre divergence in behaviour under Python 3.9.0:
+        # typing.Callable[[], foo] has __args__ = (foo,) but collections.abc.Callable
+        # has __args__ = ([], foo); and as a result is non-hashable.
+        pass
     # We also have a special case for TypeVars.
     # They are represented as instances like `~T` when they come here.
     # We need to work with their type instead.
-    if isinstance(thing, TypeVar) and type(thing) in types._global_type_lookup:  # type: ignore
+    if isinstance(thing, TypeVar) and type(thing) in types._global_type_lookup:
         return as_strategy(types._global_type_lookup[type(thing)], thing)
     # If there's no explicitly registered strategy, maybe a subtype of thing
     # is registered - if so, we can resolve it to the subclass strategy.
     # We'll start by checking if thing is from from the typing module,
     # because there are several special cases that don't play well with
     # subclass and instance checks.
-    if isinstance(thing, typing_root_type):
+    if isinstance(thing, typing_root_type) or (
+        sys.version_info[:2] >= (3, 9)
+        and isinstance(getattr(thing, "__origin__", None), type)
+        and getattr(thing, "__args__", None)
+    ):
         return types.from_typing_type(thing)
     # If it's not from the typing module, we get all registered types that are
     # a subclass of `thing` and are not themselves a subtype of any other such
@@ -1506,9 +1605,29 @@ def _from_type(thing: Type[Ex]) -> SearchStrategy[Ex]:
             "Could not resolve %r to a strategy; consider "
             "using register_type_strategy" % (thing,)
         )
-    # Finally, try to build an instance by calling the type object
+    # Finally, try to build an instance by calling the type object.  Unlike builds(),
+    # this block *does* try to infer strategies for arguments with default values.
+    # That's because of the semantic different; builds() -> "call this with ..."
+    # so we only infer when *not* doing so would be an error; from_type() -> "give
+    # me arbitrary instances" so the greater variety is acceptable.
+    # And if it's *too* varied, express your opinions with register_type_strategy()
     if not isabstract(thing):
-        return builds(thing)
+        try:
+            hints = get_type_hints(thing)
+            params = signature(thing).parameters
+        except Exception:
+            return builds(thing)
+        kwargs = {}
+        for k, p in params.items():
+            if (
+                k in hints
+                and k != "return"
+                and p.default is not Parameter.empty
+                and p.kind in (Parameter.POSITIONAL_OR_KEYWORD, Parameter.KEYWORD_ONLY)
+            ):
+                kwargs[k] = just(p.default) | _from_type(hints[k])
+        return builds(thing, **kwargs)
+    # And if it's an abstract type, we'll resolve to a union of subclasses instead.
     subclasses = thing.__subclasses__()
     if not subclasses:
         raise ResolutionFailed(
@@ -1522,10 +1641,10 @@ def _from_type(thing: Type[Ex]) -> SearchStrategy[Ex]:
 @defines_strategy(force_reusable_values=True)
 @deprecated_posargs
 def fractions(
-    min_value: Union[Real, str] = None,
-    max_value: Union[Real, str] = None,
+    min_value: Optional[Union[Real, str]] = None,
+    max_value: Optional[Union[Real, str]] = None,
     *,
-    max_denominator: int = None
+    max_denominator: Optional[int] = None,
 ) -> SearchStrategy[Fraction]:
     """Returns a strategy which generates Fractions.
 
@@ -1606,7 +1725,7 @@ def fractions(
 
 
 def _as_finite_decimal(
-    value: Union[Real, str, None], name: str, allow_infinity: Optional[bool],
+    value: Union[Real, str, None], name: str, allow_infinity: Optional[bool]
 ) -> Optional[Decimal]:
     """Convert decimal bounds to decimals, carefully."""
     assert name in ("min_value", "max_value")
@@ -1632,12 +1751,12 @@ def _as_finite_decimal(
 @defines_strategy(force_reusable_values=True)
 @deprecated_posargs
 def decimals(
-    min_value: Union[Real, str] = None,
-    max_value: Union[Real, str] = None,
+    min_value: Optional[Union[Real, str]] = None,
+    max_value: Optional[Union[Real, str]] = None,
     *,
-    allow_nan: bool = None,
-    allow_infinity: bool = None,
-    places: int = None
+    allow_nan: Optional[bool] = None,
+    allow_infinity: Optional[bool] = None,
+    places: Optional[int] = None,
 ) -> SearchStrategy[Decimal]:
     """Generates instances of :class:`python:decimal.Decimal`, which may be:
 
@@ -1723,7 +1842,7 @@ def recursive(
     base: SearchStrategy[Ex],
     extend: Callable[[SearchStrategy[Any]], SearchStrategy[T]],
     *,
-    max_leaves: int = 100
+    max_leaves: int = 100,
 ) -> SearchStrategy[Union[T, Ex]]:
     """base: A strategy to start from.
 
@@ -1805,6 +1924,12 @@ def composite(f: Callable[..., Ex]) -> Callable[..., SearchStrategy[Ex]]:
     Examples from this strategy shrink by shrinking the output of each draw
     call.
     """
+    if isinstance(f, (classmethod, staticmethod)):
+        special_method = type(f)
+        f = f.__func__
+    else:
+        special_method = None
+
     argspec = getfullargspec(f)
 
     if argspec.defaults is not None and len(argspec.defaults) == len(argspec.args):
@@ -1828,6 +1953,8 @@ def composite(f: Callable[..., Ex]) -> Callable[..., SearchStrategy[Ex]]:
         return CompositeStrategy(f, args, kwargs)
 
     accept.__module__ = f.__module__
+    if special_method is not None:
+        return special_method(accept)
     return accept
 
 
@@ -1837,9 +1964,9 @@ def composite(f: Callable[..., Ex]) -> Callable[..., SearchStrategy[Ex]]:
 def complex_numbers(
     *,
     min_magnitude: Real = 0,
-    max_magnitude: Real = None,
-    allow_infinity: bool = None,
-    allow_nan: bool = None
+    max_magnitude: Optional[Real] = None,
+    allow_infinity: Optional[bool] = None,
+    allow_nan: Optional[bool] = None,
 ) -> SearchStrategy[complex]:
     """Returns a strategy that generates complex numbers.
 
@@ -1920,7 +2047,11 @@ def complex_numbers(
 
 
 @deprecated_posargs
-def shared(base: SearchStrategy[Ex], *, key: Hashable = None) -> SearchStrategy[Ex]:
+def shared(
+    base: SearchStrategy[Ex],
+    *,
+    key: Optional[Hashable] = None,
+) -> SearchStrategy[Ex]:
     """Returns a strategy that draws a single shared value per run, drawn from
     base. Any two shared instances with the same key will share the same value,
     otherwise the identity of this strategy will be used. That is:
@@ -1943,7 +2074,7 @@ def shared(base: SearchStrategy[Ex], *, key: Hashable = None) -> SearchStrategy[
 @cacheable
 @defines_strategy(force_reusable_values=True)
 @deprecated_posargs
-def uuids(*, version: int = None) -> SearchStrategy[UUID]:
+def uuids(*, version: Optional[int] = None) -> SearchStrategy[UUID]:
     """Returns a strategy that generates :class:`UUIDs <uuid.UUID>`.
 
     If the optional version argument is given, value is passed through
@@ -2122,6 +2253,7 @@ def register_type_strategy(
             "%s will be registered for any type arguments."
             % (custom_type, origin, strategy_repr),
             since="2020-08-17",
+            has_codemod=False,
         )
         if origin in types._global_type_lookup:
             raise InvalidArgument(
@@ -2192,12 +2324,12 @@ def emails() -> SearchStrategy[str]:
 def functions(
     *,
     like: Callable[..., Any] = lambda: None,
-    returns: SearchStrategy[Any] = None,
-    pure: bool = False
+    returns: Optional[SearchStrategy[Any]] = None,
+    pure: bool = False,
 ) -> SearchStrategy[Callable[..., Any]]:
     # The proper type signature of `functions()` would have T instead of Any, but mypy
     # disallows default args for generics: https://github.com/python/mypy/issues/3737
-    """functions(*, like=lambda: None, returns=none())
+    """functions(*, like=lambda: None, returns=none(), pure=False)
 
     A strategy for functions, which can be used in callbacks.
 
